﻿//-----------------------------------------------------------------------
// <copyright file="FormSample.cs" company="emotive GmbH & Co. KG">
//     Copyright (c) emotive GmbH & Co. KG. All rights reserved.
// </copyright>
//-----------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;
using Emotive.Otx.Runtime.Api.Otx;
using Emotive.Otx.Runtime.Api.Otx.Parameter;
using Emotive.Otx.Runtime.Api.Otx.Variable;
using Emotive.Otx.Runtime.Sample;
using System.Drawing;

namespace Emotive.Otx.Runtime.Api.Sample
{
    public partial class FormSample : Form
    {
        #region Fields

        RuntimeManager rtManager = null;
        private Dictionary<Types.ScreenHandle, CustomScreenForm> customScreenDict = new Dictionary<Types.ScreenHandle, CustomScreenForm>();
        TreeNode startupNode = null;
        ImageList imageList;

        private const string ProjectImage = "ODFProject";
        private const string PackageImage = "Package";
        private const string DocumentImage = "Document";
        private const string ProcedureImage = "Procedure";
        private const string MassageBoxCaption = "OTX-Runtime-API Sample";
        private string instanceName = "Main Instance";
        private Types.DeviceServiceHandle lastDeviceServiceHandle = null;

        #endregion

        #region Constructs

        public FormSample(string instanceName)
        {
            this.instanceName = instanceName;
            Init();
        }

        public FormSample()
        {
            Init();
        }

        void Init()
        {
            rtManager = new RuntimeManager();

            InitializeComponent();

            LoadImageList();
            treeViewOtxProject.ImageList = imageList;
        }

        #endregion

        #region Window Events

        private void Form1_Load(object sender, EventArgs e)
        {
            this.comboBoxRuntimeSystems.DataSource = rtManager.DiagRuntimeSystems;
            if (this.comboBoxRuntimeSystems.Items.Count > 0)
            {
                this.comboBoxRuntimeSystems.SelectedIndex = 0;
            }

            this.Text = "emotive OTX-Runtime-API (Version " + this.rtManager.Version + ") - Sample program - " + this.instanceName + " (Thread ID: " + Thread.CurrentThread.ManagedThreadId.ToString() + ")";

            this.checkBoxWriteTraceFile.Checked = this.rtManager.IsWriteTraceFile;
            this.textBoxTraceFileFolder.Text = this.rtManager.TraceFileFolder;
            this.textBoxOutputFolder.Text = this.rtManager.OutputFolder;
            this.textBoxRdfFile.Text = this.rtManager.RdfPath;

            this.checkBoxContextVariableRead.Checked = true;
            this.checkBoxCommunicationTraceItemAdded.Checked = true;
            this.checkBoxExecutionStopped.Checked = true;
            this.checkBoxParameterChanged.Checked = true;
            this.checkBoxProcedureFinished.Checked = true;
            this.checkBoxRuntimeExceptionOccured.Checked = true;
            this.checkBoxVariableTraceItemAdded.Checked = true;
            this.checkBoxAsyncExecution.Checked = true;
            this.checkBoxAddEventsToTraceWindow.Checked = true;
            this.checkBoxValidateBeforeExecute.Checked = true;

            this.checkBoxIsCommunicationTrace.Checked = true;
            this.checkBoxIsVariablesTrace.Checked = true;

            this.gridViewParameter.Columns["dataGridViewTextBoxColumnValue"].ReadOnly = false;

            this.buttonRaiseDeviceEvent.Enabled = false;

            this.buttonBrowse.Select();
        }

        private void FormSample_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (rtManager != null)
            {
                rtManager.StopExecution();
            }
        }

        #endregion

        #region Load

        private void LoadProject()
        {
            this.UseWaitCursor = true;
            
            this.buttonLoad.Enabled = false;
            this.treeViewOtxProject.Nodes.Clear();
            this.outputControl.Rows.Clear();
            this.gridViewParameter.Rows.Clear();
            this.gridViewContext.Rows.Clear();

            if (File.Exists(this.textBoxPtxPath.Text))
            {
                try
                {
                    rtManager.Load(this.textBoxPtxPath.Text);
                }
                catch (Exception ex)
                {
                    this.DisableAllControl();
                    this.buttonLoad.Enabled = true;

                    MessageBox.Show(this, ex.Message, MassageBoxCaption, MessageBoxButtons.OK, MessageBoxIcon.Error);
                    this.UseWaitCursor = false;

                    return;
                }

                this.buttonShowSetting.Enabled = true;
                this.buttonExecute.Enabled = false;
                this.buttonValidate.Enabled = true;
                this.gridViewContext.Enabled = true;

                Package[] packages = rtManager.Packages;
                if (packages == null)
                {
                    Document doc = rtManager.GetStartUpDocument();
                    this.treeViewOtxProject.Nodes.Add(CreateDocumentNode(doc));
                    UpdateGridviewContextVariable(doc);
                }
                else
                {
                    TreeNode root = new TreeNode(rtManager.ProjectName);
                    root.ImageKey = root.SelectedImageKey = ProjectImage;
                    foreach (Package pack in packages)
                    {
                        root.Nodes.Add(this.CreatePackageNode(pack));
                    }

                    this.treeViewOtxProject.Nodes.Add(root);
                }

                treeViewOtxProject.ExpandAll();
                treeViewOtxProject.SelectedNode = startupNode;
                treeViewOtxProject.Focus();
                if (startupNode != null)
                {
                    this.buttonExecute.Enabled = true;
                    //Document doc = startupNode.Parent.Tag as Document;
                    //UpdateGridviewContextVariable(doc);
                    //UpdateGridviewParameter(startupNode.Tag as Procedure);
                }

                MessageBox.Show(this, "Load successfully!", MassageBoxCaption, MessageBoxButtons.OK, MessageBoxIcon.Information);
                this.buttonLoad.Enabled = true;
            }
            else
            {
                rtManager = null;
                this.DisableAllControl();
                this.buttonLoad.Enabled = true;
                MessageBox.Show(this, string.Format("Can not find \"{0}\"", this.textBoxPtxPath.Text), MassageBoxCaption, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

            this.UseWaitCursor = false;
        }

        private TreeNode CreateDocumentNode(Document doc)
        {
            TreeNode documentNode = new TreeNode(doc.Name);
            documentNode.Tag = doc;
            documentNode.ImageKey = documentNode.SelectedImageKey = DocumentImage;

            foreach (Procedure p in doc.Procedures)
            {
                TreeNode procedureNode = CreateProcedureNode(p);
                documentNode.Nodes.Add(procedureNode);
                if (doc.IsStartUp && p.Name == "main")
                {
                    startupNode = procedureNode;
                }
            }

            return documentNode;
        }

        private TreeNode CreateProcedureNode(Procedure procedure)
        {
            TreeNode procedureNode = new TreeNode(procedure.Name);
            procedureNode.ImageKey = procedureNode.SelectedImageKey = ProcedureImage;
            procedureNode.Tag = procedure;
            return procedureNode;
        }

        private TreeNode CreatePackageNode(Package pack)
        {
            TreeNode packageNode = new TreeNode(pack.Name);
            packageNode.Tag = pack;
            packageNode.ImageKey = packageNode.SelectedImageKey = PackageImage;

            List<Document> documents = pack.Documents.ToList();
            foreach (Document doc in documents)
            {
                packageNode.Nodes.Add(CreateDocumentNode(doc));
            }

            foreach (Package package in pack.Packages)
            {
                packageNode.Nodes.Add(this.CreatePackageNode(package));
            }

            return packageNode;
        }

        #endregion

        #region Action Events

        private void buttonExecute_Click(object sender, EventArgs e)
        {
            this.buttonRaiseDeviceEvent.Enabled = false;
            
            if (rtManager != null)
            {
                System.Collections.Generic.List<Error> errors = null;
                if (this.CanExecute(ref errors))
                {
                    this.textBoxPtxPath.Enabled = false;
                    this.buttonBrowse.Enabled = false;

                    if (this.checkBoxProcedureFinished.Checked && this.checkBoxExecutionStopped.Checked && this.checkBoxRuntimeExceptionOccured.Checked)
                    {
                        this.DisableAllControl();
                    }

                    this.buttonStop.Enabled = true;
                    
                    this.outputControl.Rows.Clear();

                    rtManager.DiagRuntimeSystem = this.comboBoxRuntimeSystems.Text.Trim();
                    if (rtManager.DiagRuntimeSystem != null)
                    {
                        this.comboBoxRuntimeSystems.Enabled = false;
                        this.textBoxRdfFile.Enabled = false;
                        this.buttonBrowseRdfFile.Enabled = false;
                    }

                    if (treeViewOtxProject.SelectedNode != null && treeViewOtxProject.SelectedNode.Tag is Procedure)
                    {
                        Procedure procedure = treeViewOtxProject.SelectedNode.Tag as Procedure;
                        this.AdjustValueParam(procedure);

                        if (this.checkBoxAsyncExecution.Checked)
                        {
                            procedure.Execute(true);
                        }
                        else
                        {
                            Task task = new Task(() =>
                            {
                                procedure.Execute(false);
                            });

                            task.Start(TaskScheduler.FromCurrentSynchronizationContext());
                        }
                    }
                }
            }
        }

        private void buttonLoad_Click(object sender, EventArgs e)
        {
            this.LoadProject();
        }

        private void buttonValidate_Click(object sender, EventArgs e)
        {
            try
            {
                if (rtManager != null)
                {
                    rtManager.DiagRuntimeSystem = this.comboBoxRuntimeSystems.Text.Trim();
                    if (rtManager.DiagRuntimeSystem != null)
                    {
                        this.comboBoxRuntimeSystems.Enabled = false;
                        this.textBoxRdfFile.Enabled = false;
                        this.buttonBrowseRdfFile.Enabled = false;
                    }

                    System.Collections.Generic.List<Error> errors = rtManager.Validate().ToList();

                    if (errors.Count > 0)
                    {
                        FormValidate form = new FormValidate(errors);
                        form.ShowDialog();
                    }
                    else
                    {
                        MessageBox.Show(this, "All documents are validated successfully without any errors or warnings.", MassageBoxCaption, MessageBoxButtons.OK, MessageBoxIcon.Information);
                    }
                }
            }
            catch (Exception ex)
            {
                this.DisplayException(ex);
            }
        }

        private void buttonStop_Click(object sender, EventArgs e)
        {
            this.buttonRaiseDeviceEvent.Enabled = false;
            
            if (rtManager != null)
            {
                rtManager.StopExecution();
            }
        }

        private void buttonNewInstance_Click(object sender, EventArgs e)
        {
            FormSample form = new FormSample("Child Instance");
            form.Show();
        }

        private void buttonNewInstanceNewThread_Click(object sender, EventArgs e)
        {
            Thread thread = new Thread(new ThreadStart(NewInstanceInThread));
            thread.SetApartmentState(ApartmentState.STA);
            thread.Start();
        }

        private void NewInstanceInThread()
        {
            Application.Run(new FormSample("Main Instance"));
        }

        private void buttonShowSetting_Click(object sender, EventArgs e)
        {
            if (rtManager != null)
            {
                rtManager.DiagRuntimeSystem = this.comboBoxRuntimeSystems.Text.Trim();
                if (rtManager.DiagRuntimeSystem != null)
                {
                    this.comboBoxRuntimeSystems.Enabled = false;
                    this.textBoxRdfFile.Enabled = false;
                    this.buttonBrowseRdfFile.Enabled = false;
                }

                Setting.ProjectOptionsForm form = new Setting.ProjectOptionsForm(rtManager.ProjectSettings, rtManager);
                form.ShowDialog();
            }
        }

        private void buttonBrowse_Click(object sender, EventArgs e)
        {
            OpenFileDialog fileDialog = new OpenFileDialog();

            string dataDirectory = Path.GetFullPath(@"..\..\Data");
            if (string.IsNullOrEmpty(this.textBoxPtxPath.Text) && Directory.Exists(dataDirectory))
            {
                fileDialog.InitialDirectory = dataDirectory;
            }

            fileDialog.RestoreDirectory = true;
            fileDialog.Title = "Please select a PTX/OTX file";
            fileDialog.Filter = "All supported formats(*.ptx, *.otx, *.zip)|*.ptx;*.otx;*.zip|OTX Files(*.otx)|*.otx|PTX Files(*.ptx)|*.ptx|All Files (*.*)|*.*";
            fileDialog.Multiselect = false;
            if (fileDialog.ShowDialog() == DialogResult.OK)
            {
                this.textBoxPtxPath.Text = fileDialog.FileName;
                this.LoadProject();
            }
        }

        private void buttonRdfFileBrowse_Click(object sender, EventArgs e)
        {
            OpenFileDialog fileDialog = new OpenFileDialog();

            if (Directory.Exists(Path.GetDirectoryName(this.textBoxRdfFile.Text)))
            {
                fileDialog.InitialDirectory = Path.GetDirectoryName(this.textBoxRdfFile.Text);
            }

            fileDialog.RestoreDirectory = true;
            fileDialog.Title = "Please select the RDF file for MVCI-Server Configuration";
            fileDialog.Filter = "All supported formats(*.rdf, *.xml)|*.rdf;*.xml|RDF Files(*.rdf)|*.rdf|XML Files(*.xml)|*.xml|All Files (*.*)|*.*";
            fileDialog.Multiselect = false;
            if (fileDialog.ShowDialog() == DialogResult.OK)
            {
                this.textBoxRdfFile.Text = fileDialog.FileName;
            }
        }

        private void checkBoxAddEventsToTraceWindow_CheckedChanged(object sender, EventArgs e)
        {
            this.outputControl.Enabled = this.checkBoxAddEventsToTraceWindow.Checked;
            this.outputControl.BackgroundColor = this.checkBoxAddEventsToTraceWindow.Checked ? SystemColors.Window : SystemColors.Control;
        }

        private void checkBoxIsCommunicationTrace_CheckedChanged(object sender, EventArgs e)
        {
            this.rtManager.IsCommunicationTrace = this.checkBoxIsCommunicationTrace.Checked;
        }

        private void checkBoxIsVariablesTrace_CheckedChanged(object sender, EventArgs e)
        {
            this.rtManager.IsVariablesTrace = this.checkBoxIsVariablesTrace.Checked;
        }

        #endregion

        #region Runtime Manager Events

        void rtManager_RuntimeExceptionOccured(Procedure procedure, Exception exception)
        {
            this.AddOutputRow("RuntimeExceptionOccured(" + procedure.Name + ", " + exception.Message + ")");

            this.DisplayException(exception);

            this.EnableAllControl();
            this.buttonStop.Enabled = false;
            this.textBoxPtxPath.Enabled = true;
            this.buttonBrowse.Enabled = true;
            this.buttonRaiseDeviceEvent.Enabled = false;

            this.CloseAllCustomScreens();
        }

        void rtManager_ExecutionStopped()
        {
            this.AddOutputRow("ExecutionStopped()");

            this.EnableAllControl();
            this.buttonStop.Enabled = false;
            this.textBoxPtxPath.Enabled = true;
            this.buttonBrowse.Enabled = true;
            this.buttonRaiseDeviceEvent.Enabled = false;


            this.CloseAllCustomScreens();
        }

        void rtManager_ProcedureFinished(Procedure procedure)
        {
            this.AddOutputRow("ProcedureFinished(" + procedure.Name + ")");

            this.EnableAllControl();
            this.buttonStop.Enabled = false;
            this.textBoxPtxPath.Enabled = true;
            this.buttonBrowse.Enabled = true;
            this.buttonRaiseDeviceEvent.Enabled = false;

            this.CloseAllCustomScreens();

            this.UpdateGridviewParameter(procedure);
        }

        void rtManager_ContextVariableRead(Emotive.Otx.Runtime.Api.Otx.Variable.ContextVariable contextVariable, ref object value)
        {
            foreach (DataGridViewRow row in this.gridViewContext.Rows)
            {
                if ((row.Cells[0].Value != null) && (row.Cells[0].Value.ToString() == contextVariable.Name) && (row.Cells[2] != null))
                {
                    value = row.Cells[2].Value;
                    break;
                }
            }

            this.AddOutputRow("ContextVariableRead(" + contextVariable.Name + ", " + (value != null ? value.ToString() : null) + ")");
        }

        void rtManager_ParameterValueChanged(Emotive.Otx.Runtime.Api.Otx.Parameter.Parameter parameter, Procedure procedure)
        {
            this.AddOutputRow("ParameterValueChanged(" + procedure.Name + ", " + parameter.Name + ", " + (parameter is InOutParameter ? (parameter as InOutParameter).Value.ToString() : (parameter as OutParameter).Value.ToString()) + ")");
            this.UpdateGridviewParameter(procedure);
        }

        void rtManager_VariableTraceItemAdded(VariableTraceItem traceItem)
        {
            this.AddOutputRow("VariableTraceItemAdded(" + traceItem.Timestamp + ", " + traceItem.Document + ", " + traceItem.Procedure + ", " + traceItem.Variable + ", " + traceItem.Value + ")");
        }

        void rtManager_CommunicationTraceItemAdded(CommunicationTraceItem traceItem)
        {
            this.AddOutputRow("CommunicationTraceItemAdded(" + traceItem.Traffic + ", " + traceItem.Name + ", " + traceItem.Timestamp + ", " + traceItem.Value + ", " + traceItem.DataType + ", " + traceItem.Pdu + ", " + traceItem.Error + ")");
        }

        void rtManager_WriteLogPerformed(Types.SeverityLevels logLevel, string target, string message, string rawMessage, bool isImplicit)
        {
            this.AddOutputRow("WriteLogPerformed(" + Enum.GetName(typeof(Types.SeverityLevels), logLevel) + ", " + target + ", " + message + ", " + rawMessage + ", " + isImplicit.ToString() + ")");
        }

        void rtManager_ConfirmDialogOccurred(string title, string message, Types.MessageTypes messageType, out Types.ConfirmationTypes result)
        {
            result = Types.ConfirmationTypes.YES;
            MessageBoxIcon icon;
            MessageBoxButtons button;
            switch (messageType)
            {
                case Types.MessageTypes.INFO:
                    icon = MessageBoxIcon.Information;
                    button = MessageBoxButtons.OK;
                    break;
                case Types.MessageTypes.WARNING:
                    icon = MessageBoxIcon.Warning;
                    button = MessageBoxButtons.OK;
                    break;
                case Types.MessageTypes.ERROR:
                    icon = MessageBoxIcon.Error;
                    button = MessageBoxButtons.OK;
                    break;
                case Types.MessageTypes.YESNO_QUESTION:
                    icon = MessageBoxIcon.Question;
                    button = MessageBoxButtons.YesNo;
                    break;
                case Types.MessageTypes.YESNOCANCEL_QUESTION:
                    icon = MessageBoxIcon.Question;
                    button = MessageBoxButtons.YesNoCancel;
                    break;
                default:
                    icon = MessageBoxIcon.Information;
                    button = MessageBoxButtons.OK;
                    break;
            }

            switch (MessageBox.Show(this, "Self generated ConfirmDialog:\n\n" + message, title, button, icon))
            {
                case System.Windows.Forms.DialogResult.OK:
                case System.Windows.Forms.DialogResult.Yes:
                    result = Types.ConfirmationTypes.YES;
                    break;
                case System.Windows.Forms.DialogResult.No:
                    result = Types.ConfirmationTypes.NO;
                    break;
                case System.Windows.Forms.DialogResult.Cancel:
                    result = Types.ConfirmationTypes.CANCEL;
                    break;
            }

            this.AddOutputRow(string.Format("ConfirmDialogOccurred({0}, {1}, {2}, out {3})", title, message, Enum.GetName(typeof(Types.MessageTypes), messageType), Enum.GetName(typeof(Types.ConfirmationTypes), result)));
        }

        void rtManager_InputDialogOccurred(string title, string message, string initialValue, string instruction, string restriction, out string result)
        {
            result = "emotive";
            this.AddOutputRow(string.Format("InputDialogOccurred({0}, {1}, {2}, {3}, {4})", title, message, initialValue, instruction, restriction));
        }

        void rtManager_ChoiceDialogOccurred(string title, string message, string[] options, int @default, out int result)
        {
            string strOptions = string.Empty;
            if (options != null)
            {
                foreach (string str in options)
                {
                    strOptions += "\"" + str + "\", ";
                }
            }

            string strDefault = string.Empty;
            if (options != null && options.Length >= @default)
            {
                strDefault = options[@default];
            }

            result = 4;

            this.AddOutputRow(string.Format("ChoiceDialogOccurred({0}, {1}, {2}, {3}, out {4})", title, message, strOptions, strDefault, result.ToString()));
        }

        void rtManager_ShowDocumentDialogOccurred(string title, string message, string document)
        {
            this.AddOutputRow(string.Format("ShowDocumentDialogOccurred({0}, {1}, {2})", title, message, document));
        }

        void rtManager_ExecuteDeviceServiceOccurred(Types.DeviceServiceHandle handle, Boolean executeAsync, Boolean suppressInvalidMeasurementException, List<DeviceServiceParameter> deviceServiceParameters)
        {
            this.buttonRaiseDeviceEvent.Enabled = true;
            this.lastDeviceServiceHandle = handle;
            string strParameters = string.Empty;
            if (deviceServiceParameters != null)
            {
                foreach (Parameter parameter in deviceServiceParameters)
                {
                    if (parameter is DeviceServiceOutParameter)
                    {
                        (parameter as DeviceServiceOutParameter).Value = 11;
                        
                    }

                    strParameters += "(" + parameter.Direction.ToString() + ") " + parameter.Name + ", ";
                }
            }

            strParameters = strParameters.Substring(0, strParameters.LastIndexOf(','));

            this.AddOutputRow(string.Format("ExecuteDeviceServiceOccurred({0}, {1}, {2}, {3}, {4})", new object[] { handle.DeviceSignatureName, handle.DeviceServiceName, executeAsync, suppressInvalidMeasurementException, strParameters }));
        }

        void rtManager_OpenScreenOccurred(string screen, bool modal, ScreenParameter[] screenParameters, out Types.ScreenHandle screenHandle)
        {
            string strParameters = string.Empty;
            if (screenParameters != null)
            {
                foreach (ScreenParameter screenParameter in screenParameters)
                {
                    strParameters += "(" + screenParameter.Direction.ToString() + ") " + screenParameter.Name + ", ";
                }
            }

            strParameters = strParameters.Substring(0, strParameters.LastIndexOf(','));
            screenHandle = new Types.ScreenHandle();

            this.AddOutputRow(string.Format("OpenScreenOccurred({0}, {1}, {2}, out new ScreenHandle())", screen, modal, strParameters));

            CustomScreenForm customScreen = new CustomScreenForm(screen, modal, screenParameters, screenHandle);

            customScreen.Show();
            customScreen.BringToFront();
            customScreen.FormClosed += new FormClosedEventHandler(customScreen_FormClosed);

            this.customScreenDict.Add(screenHandle, customScreen);
        }

        void customScreen_FormClosed(object sender, FormClosedEventArgs e)
        {
            CustomScreenForm customScreenForm = sender as CustomScreenForm;
            if (customScreenForm != null)
            {
                if (customScreenForm != null && !customScreenForm.IsDisposed)
                {
                    this.rtManager.ScreenIsClosed(customScreenForm.ScreenHandle);
                }
            }
        }

        void rtManager_ScreenParameterValueChanged(ScreenParameter screenParameter, Types.ScreenHandle screenHandle)
        {
            string strParameters = string.Empty;

            object value = null;
            if (screenParameter is ScreenInParameter)
            {
                value = (screenParameter as ScreenInParameter).Value;
            }
            else if (screenParameter is ScreenInOutParameter)
            {
                value = (screenParameter as ScreenInOutParameter).Value;
            }

            this.AddOutputRow(string.Format("ScreenParameterValueChanged({0}, screenHandle)", "(" + screenParameter.Direction.ToString() + "," + value + ")" + screenParameter.Name));

            if (this.customScreenDict.ContainsKey(screenHandle) && ((screenParameter is ScreenInParameter) || (screenParameter is ScreenInOutParameter)))
            {
                this.customScreenDict[screenHandle].UpdateParameter(screenParameter);
            }
        }

        void rtManager_HighlightScreenOccurred(Types.ScreenHandle screenHandle)
        {
            this.AddOutputRow("HighlightScreenOccurred(screenHandle)");

            if (this.customScreenDict.ContainsKey(screenHandle))
            {
                CustomScreenForm customScreenForm = this.customScreenDict[screenHandle];
                if (customScreenForm != null && !customScreenForm.IsDisposed)
                {
                    customScreenForm.BringToFront();
                }
            }
        }

        void rtManager_ScreenIsOpenOccurred(Types.ScreenHandle screenHandle, out bool isOpened)
        {
            isOpened = false;

            if (this.customScreenDict.ContainsKey(screenHandle))
            {
                CustomScreenForm customScreenForm = this.customScreenDict[screenHandle];
                if (customScreenForm != null && !customScreenForm.IsDisposed)
                {
                    isOpened = customScreenForm.Visible;
                }
            }

            this.AddOutputRow(string.Format("ScreenIsOpenOccurred(screenHandle, out {0}", isOpened.ToString()));
        }

        void rtManager_CloseScreenOccurred(Types.ScreenHandle screenHandle)
        {
            this.AddOutputRow("CloseScreenOccurred(screenHandle)");

            if (this.customScreenDict.ContainsKey(screenHandle))
            {
                CustomScreenForm customScreenForm = this.customScreenDict[screenHandle];
                if (customScreenForm != null && !customScreenForm.IsDisposed)
                {
                    customScreenForm.Close();
                }
            }
        }

        private void checkBoxVariablesTrace_CheckedChanged(object sender, EventArgs e)
        {
            if (this.checkBoxVariableTraceItemAdded.Checked)
            {
                rtManager.VariableTraceItemAdded += new RuntimeManager.VariableTraceItemAddedHandler(rtManager_VariableTraceItemAdded);
            }
            else
            {
                rtManager.VariableTraceItemAdded -= new RuntimeManager.VariableTraceItemAddedHandler(rtManager_VariableTraceItemAdded);
            }
        }

        private void checkBoxCommunicationTrace_CheckedChanged(object sender, EventArgs e)
        {
            if (this.checkBoxCommunicationTraceItemAdded.Checked)
            {
                rtManager.CommunicationTraceItemAdded += new RuntimeManager.CommunicationTraceItemAddedHandler(rtManager_CommunicationTraceItemAdded);
            }
            else
            {
                rtManager.CommunicationTraceItemAdded -= new RuntimeManager.CommunicationTraceItemAddedHandler(rtManager_CommunicationTraceItemAdded);
            }
        }

        private void checkBoxParameterChanged_CheckedChanged(object sender, EventArgs e)
        {
            if (this.checkBoxParameterChanged.Checked)
            {
                rtManager.ParameterValueChanged += new RuntimeManager.ParameterValueChangedHandler(rtManager_ParameterValueChanged);
            }
            else
            {
                rtManager.ParameterValueChanged -= new RuntimeManager.ParameterValueChangedHandler(rtManager_ParameterValueChanged);
            }
        }

        private void checkBoxContextVariableRead_CheckedChanged(object sender, EventArgs e)
        {
            if (this.checkBoxContextVariableRead.Checked)
            {
                rtManager.ContextVariableRead += new RuntimeManager.ContextVariableReadHandler(rtManager_ContextVariableRead);
            }
            else
            {
                rtManager.ContextVariableRead -= new RuntimeManager.ContextVariableReadHandler(rtManager_ContextVariableRead);
            }
        }

        private void checkBoxExecutionStopped_CheckedChanged(object sender, EventArgs e)
        {
            if (this.checkBoxExecutionStopped.Checked)
            {
                rtManager.ExecutionStopped += new RuntimeManager.ExecutionStoppedHandler(rtManager_ExecutionStopped);
            }
            else
            {
                rtManager.ExecutionStopped -= new RuntimeManager.ExecutionStoppedHandler(rtManager_ExecutionStopped);
            }
        }

        private void checkBoxProcedureFinished_CheckedChanged(object sender, EventArgs e)
        {
            if (this.checkBoxProcedureFinished.Checked)
            {
                rtManager.ProcedureFinished += new RuntimeManager.ProcedureFinishedHandler(rtManager_ProcedureFinished);
            }
            else
            {
                rtManager.ProcedureFinished -= new RuntimeManager.ProcedureFinishedHandler(rtManager_ProcedureFinished);
            }
        }

        private void checkBoxRuntimeExceptionOccured_CheckedChanged(object sender, EventArgs e)
        {
            if (this.checkBoxRuntimeExceptionOccured.Checked)
            {
                rtManager.RuntimeExceptionOccured += new RuntimeManager.RuntimeExceptionOccuredHandler(rtManager_RuntimeExceptionOccured);
            }
            else
            {
                rtManager.RuntimeExceptionOccured -= new RuntimeManager.RuntimeExceptionOccuredHandler(rtManager_RuntimeExceptionOccured);
            }
        }

        #endregion

        #region Control Events

        private void checkBoxWriteTraceFile_CheckedChanged(object sender, EventArgs e)
        {
            if (rtManager != null)
            {
                rtManager.IsWriteTraceFile = checkBoxWriteTraceFile.Checked;
            }
        }

        private void buttonBrowseTraceFileFolder_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog dialog = new FolderBrowserDialog();
            if (dialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                this.textBoxTraceFileFolder.Text = dialog.SelectedPath;
                if (this.rtManager != null)
                {
                    this.rtManager.TraceFileFolder = this.textBoxTraceFileFolder.Text;
                }
            }
        }

        private void buttonBrowseOutputFolder_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog dialog = new FolderBrowserDialog();
            if (dialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                this.textBoxOutputFolder.Text = dialog.SelectedPath;
                if (this.rtManager != null)
                {
                    this.rtManager.OutputFolder = this.textBoxOutputFolder.Text;
                }
            }
        }

        private void textBoxTraceFileFolder_Leave(object sender, EventArgs e)
        {
            if (this.rtManager != null)
            {
                this.rtManager.TraceFileFolder = this.textBoxTraceFileFolder.Text;
            }
        }

        private void textBoxOutputFolder_Leave(object sender, EventArgs e)
        {
            if (this.rtManager != null)
            {
                this.rtManager.OutputFolder = this.textBoxOutputFolder.Text;
            }
        }

        private void checkBoxRegisterAllEvents_CheckedChanged(object sender, EventArgs e)
        {
            bool registered = this.checkBoxRegisterAllEvents.Checked;

            this.checkBoxChoiceDialog.Checked = registered;
            this.checkBoxCloseScreen.Checked = registered;
            this.checkBoxConfirmDialog.Checked = registered;
            this.checkBoxHighlightScreen.Checked = registered;
            this.checkBoxInputDialog.Checked = registered;
            this.checkBoxOpenScreen.Checked = registered;
            this.checkBoxScreenIsOpen.Checked = registered;
            this.checkBoxScreenParameterValueChanged.Checked = registered;
            this.checkBoxShowDocumentDialog.Checked = registered;
            this.checkBoxWriteLogPerformed.Checked = registered;
            this.checkBoxContextVariableRead.Checked = registered;
            this.checkBoxCommunicationTraceItemAdded.Checked = registered;
            this.checkBoxExecutionStopped.Checked = registered;
            this.checkBoxParameterChanged.Checked = registered;
            this.checkBoxProcedureFinished.Checked = registered;
            this.checkBoxRuntimeExceptionOccured.Checked = registered;
            this.checkBoxVariableTraceItemAdded.Checked = registered;
            this.checkBoxExecuteDeviceServiceOccured.Checked = registered;
        }

        private void checkBoxWriteLogPerformed_CheckedChanged(object sender, EventArgs e)
        {
            if (this.checkBoxWriteLogPerformed.Checked)
            {
                this.rtManager.WriteLogPerformed += new RuntimeManager.WriteLogPerformedHandler(rtManager_WriteLogPerformed);
            }
            else
            {
                this.rtManager.WriteLogPerformed -= new RuntimeManager.WriteLogPerformedHandler(rtManager_WriteLogPerformed);
            }
        }

        private void checkBoxConfirmDialog_CheckedChanged(object sender, EventArgs e)
        {
            if (this.checkBoxConfirmDialog.Checked)
            {
                this.rtManager.ConfirmDialogOccurred += new RuntimeManager.ConfirmDialogOccurredHandler(rtManager_ConfirmDialogOccurred);
            }
            else
            {
                this.rtManager.ConfirmDialogOccurred -= new RuntimeManager.ConfirmDialogOccurredHandler(rtManager_ConfirmDialogOccurred);
            }
        }

        private void checkBoxInputDialog_CheckedChanged(object sender, EventArgs e)
        {
            if (this.checkBoxInputDialog.Checked)
            {
                this.rtManager.InputDialogOccurred += new RuntimeManager.InputDialogOccurredHandler(rtManager_InputDialogOccurred);
            }
            else
            {
                this.rtManager.InputDialogOccurred -= new RuntimeManager.InputDialogOccurredHandler(rtManager_InputDialogOccurred);
            }
        }

        private void checkBoxChoiceDialog_CheckedChanged(object sender, EventArgs e)
        {
            if (this.checkBoxChoiceDialog.Checked)
            {
                this.rtManager.ChoiceDialogOccurred += new RuntimeManager.ChoiceDialogOccurredHandler(rtManager_ChoiceDialogOccurred);
            }
            else
            {
                this.rtManager.ChoiceDialogOccurred -= new RuntimeManager.ChoiceDialogOccurredHandler(rtManager_ChoiceDialogOccurred);
            }
        }

        private void checkBoxShowDocumentDialog_CheckedChanged(object sender, EventArgs e)
        {
            if (this.checkBoxShowDocumentDialog.Checked)
            {
                this.rtManager.ShowDocumentDialogOccurred += new RuntimeManager.ShowDocumentDialogOccurredHandler(rtManager_ShowDocumentDialogOccurred);
            }
            else
            {
                this.rtManager.ShowDocumentDialogOccurred -= new RuntimeManager.ShowDocumentDialogOccurredHandler(rtManager_ShowDocumentDialogOccurred);
            }
        }

        private void checkBoxOpenScreen_CheckedChanged(object sender, EventArgs e)
        {
            if (this.checkBoxOpenScreen.Checked)
            {
                this.rtManager.OpenScreenOccurred += new RuntimeManager.OpenScreenOccurredHandler(rtManager_OpenScreenOccurred);
            }
            else
            {
                this.rtManager.OpenScreenOccurred -= new RuntimeManager.OpenScreenOccurredHandler(rtManager_OpenScreenOccurred);
            }
        }

        private void checkBoxScreenParameterValueChanged_CheckedChanged(object sender, EventArgs e)
        {
            if (this.checkBoxScreenParameterValueChanged.Checked)
            {
                this.rtManager.ScreenParameterValueChanged += new RuntimeManager.ScreenParameterValueChangedHandler(rtManager_ScreenParameterValueChanged);
            }
            else
            {
                this.rtManager.ScreenParameterValueChanged -= new RuntimeManager.ScreenParameterValueChangedHandler(rtManager_ScreenParameterValueChanged);
            }
        }

        private void checkBoxHighlightScreen_CheckedChanged(object sender, EventArgs e)
        {
            if (this.checkBoxHighlightScreen.Checked)
            {
                this.rtManager.HighlightScreenOccurred += new RuntimeManager.HighlightScreenOccurredHandler(rtManager_HighlightScreenOccurred);
            }
            else
            {
                this.rtManager.HighlightScreenOccurred -= new RuntimeManager.HighlightScreenOccurredHandler(rtManager_HighlightScreenOccurred);
            }
        }

        private void checkBoxScreenIsOpen_CheckedChanged(object sender, EventArgs e)
        {
            if (this.checkBoxScreenIsOpen.Checked)
            {
                this.rtManager.ScreenIsOpenOccurred += new RuntimeManager.ScreenIsOpenOccurredHandler(rtManager_ScreenIsOpenOccurred);
            }
            else
            {
                this.rtManager.ScreenIsOpenOccurred -= new RuntimeManager.ScreenIsOpenOccurredHandler(rtManager_ScreenIsOpenOccurred);
            }
        }

        private void checkBoxCloseScreen_CheckedChanged(object sender, EventArgs e)
        {
            if (this.checkBoxCloseScreen.Checked)
            {
                this.rtManager.CloseScreenOccurred += new RuntimeManager.CloseScreenOccurredHandler(rtManager_CloseScreenOccurred);
            }
            else
            {
                this.rtManager.CloseScreenOccurred -= new RuntimeManager.CloseScreenOccurredHandler(rtManager_CloseScreenOccurred);
            }
        }

        private void treeViewOtxProject_AfterSelect(object sender, TreeViewEventArgs e)
        {
            TreeNode node = e.Node;
            if (node != null)
            {
                this.gridViewParameter.Rows.Clear();
                if (node.Tag is Procedure)
                {
                    this.buttonExecute.Enabled = true;
                    UpdateGridviewParameter(node.Tag as Procedure);
                    Document doc = node.Parent.Tag as Document;
                    UpdateGridviewContextVariable(doc);
                }
                else
                {
                    this.buttonExecute.Enabled = false;
                }
            }
        }

        private void textBoxPtxPath_TextChanged(object sender, EventArgs e)
        {
            if (File.Exists(this.textBoxPtxPath.Text))
            {
                this.buttonLoad.Enabled = true;
                this.rtManager.RdfPath = this.textBoxRdfFile.Text;
            }
            else
            {
                this.DisableAllControl();
            }
        }

        private void CheckBoxExecuteDeviceServiceOccured_CheckedChanged(object sender, EventArgs e)
        {
            if (this.checkBoxExecuteDeviceServiceOccured.Checked)
            {
                this.rtManager.ExecuteDeviceServiceOccurred += new RuntimeManager.ExecuteDeviceServiceOccurredHandler(rtManager_ExecuteDeviceServiceOccurred);
            }
            else
            {
                this.rtManager.ExecuteDeviceServiceOccurred -= new RuntimeManager.ExecuteDeviceServiceOccurredHandler(rtManager_ExecuteDeviceServiceOccurred);
            }
        }

        private void buttonRaiseDeviceEvent_Click(object sender, EventArgs e)
        {
            if (this.lastDeviceServiceHandle != null)
            {
                this.rtManager.RaiseDeviceEvent(this.lastDeviceServiceHandle, string.Empty, string.Empty);
            }
        }

        #endregion

        #region Helper

        void AddOutputRow(string text)
        {
            if (this.checkBoxAddEventsToTraceWindow.Checked)
            {
                object[] values = new object[] { DateTime.Now.ToString("hh:mm:ss.fff"), text };
                int index = this.outputControl.Rows.Add(values);
                this.outputControl.FirstDisplayedScrollingRowIndex = index;
            }
        }

        private void DisableAllControl()
        {
            this.buttonLoad.Enabled = false;
            this.buttonShowSetting.Enabled = false;
            this.buttonExecute.Enabled = false;
            this.buttonValidate.Enabled = false;
            this.gridViewContext.Enabled = false;
        }

        private void EnableAllControl()
        {
            this.buttonLoad.Enabled = true;
            this.buttonShowSetting.Enabled = true;
            this.buttonValidate.Enabled = true;
            this.gridViewContext.Enabled = true;

            if (this.treeViewOtxProject.SelectedNode != null && this.treeViewOtxProject.SelectedNode.Tag is Procedure)
            {
                this.buttonExecute.Enabled = true;
            }
        }

        private bool CanExecute(ref System.Collections.Generic.List<Error> errors)
        {
            if (this.checkBoxValidateBeforeExecute.Checked)
            {
                try
                {
                    if (rtManager != null)
                    {
                        errors = rtManager.Validate().ToList();
                        foreach (Error err in errors)
                        {
                            if (err.Severity == SeverityType.Critical)
                            {
                                MessageBox.Show(this, "To run a procedure make sure, that all errors shown in form list are corrected.", MassageBoxCaption, MessageBoxButtons.OK, MessageBoxIcon.Error);

                                FormValidate form = new FormValidate(errors);
                                form.ShowDialog();

                                return false;
                            }
                        }
                    }
                }
                catch (Exception e)
                {
                    this.DisplayException(e);
                    return false;
                }
            }

            return true;
        }

        void DisplayException(Exception e)
        {
            MessageBox.Show(this, e.Message, MassageBoxCaption + " - Exception occured", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }

        private int GetIndexParam(Procedure procedure, string name)
        {
            for (int i = 0; i < procedure.Parameters.Length; i++)
            {
                if (procedure.Parameters[i].Name.Equals(name.Trim()))
                {
                    return i;
                }
            }

            return -1;
        }

        private void AdjustValueParam(Procedure procedure)
        {
            if (this.gridViewParameter.Rows.Count > 0)
            {
                for (int i = 0; i < this.gridViewParameter.Rows.Count; i++)
                {
                    if (this.gridViewParameter.Rows[i].Cells[0].Value != null)
                    {
                        int index = this.GetIndexParam(procedure, this.gridViewParameter.Rows[i].Cells[0].Value.ToString());
                        if (index == -1)
                        {
                            continue;
                        }

                        object value = this.gridViewParameter.Rows[i].Cells[2].Value;
                        if (procedure.Parameters[index].DataType.Name.Equals("Boolean"))
                        {
                            try
                            {
                                value = bool.Parse(value.ToString());
                            }
                            catch
                            {
                            }
                        }
                        else if (procedure.Parameters[index].DataType.Name.Equals("Integer"))
                        {
                            try
                            {
                                value = Int64.Parse(value.ToString());
                            }
                            catch
                            {
                            }
                        }
                        else if (procedure.Parameters[index].DataType.Name.Equals("Float"))
                        {
                            try
                            {
                                value = Double.Parse(value.ToString());
                            }
                            catch
                            {
                            }
                        }

                        if (procedure.Parameters[index] is InParameter)
                        {
                            if ((procedure.Parameters[index] as InParameter).Value != value)
                            {
                                (procedure.Parameters[index] as InParameter).Value = value;
                            }
                        }
                        else if (procedure.Parameters[index] is InOutParameter)
                        {
                            if ((procedure.Parameters[index] as InOutParameter).Value != value)
                            {
                                (procedure.Parameters[index] as InOutParameter).Value = value;
                            }
                        }
                    }
                }
            }
        }

        private void LoadImageList()
        {
            imageList = new ImageList();
            imageList.Images.Add(ProjectImage, global::Emotive.Otx.Runtime.Sample.Properties.Resources.ODFProject);
            imageList.Images.Add(PackageImage, global::Emotive.Otx.Runtime.Sample.Properties.Resources.Package);
            imageList.Images.Add(DocumentImage, global::Emotive.Otx.Runtime.Sample.Properties.Resources.DocumentOTX16);
            imageList.Images.Add(ProcedureImage, global::Emotive.Otx.Runtime.Sample.Properties.Resources.Procedure);
        }

        private void UpdateGridviewContextVariable(Document doc)
        {
            if (doc != this.gridViewContext.Tag as Document)
            {
                this.gridViewContext.Rows.Clear();
                this.gridViewContext.Tag = doc;
                if (doc == null || doc.ContextVariables == null)
                {
                    return;
                }

                object[] values = null;
                foreach (ContextVariable context in doc.ContextVariables)
                {
                    values = new object[] { context.Name, context.DataType.Name };
                    this.gridViewContext.Rows.Add(values);
                }
            }
        }

        private void UpdateGridviewParameter(Procedure procedure)
        {
            if (procedure == null || procedure.Parameters == null)
            {
                return;
            }

            object[] values = null;
            foreach (Parameter param in procedure.Parameters)
            {
                if (param.DataType.Name.Equals("Boolean") || param.DataType.Name.Equals("Integer") || param.DataType.Name.Equals("Float") || param.DataType.Name.Equals("String"))
                {
                    bool bValueWasSet = false;

                    foreach (DataGridViewRow row in this.gridViewParameter.Rows)
                    {
                        if (row.Cells["dataGridViewTextBoxColumnName"].Value.Equals(param.Name))
                        {
                            if (param is InParameter)
                            {
                                row.Cells["dataGridViewTextBoxColumnValue"].Value = (param as InParameter).Value;
                            }
                            else if (param is InOutParameter)
                            {
                                row.Cells["dataGridViewTextBoxColumnValue"].Value = (param as InOutParameter).Value;
                            }
                            else if (param is OutParameter)
                            {
                                row.Cells["dataGridViewTextBoxColumnValue"].Value = (param as OutParameter).Value;
                            }

                            bValueWasSet = true;
                            break;
                        }
                    }

                    if (!bValueWasSet)
                    {
                        if (param is InParameter)
                        {
                            values = new object[] { param.Name, param.DataType.Name, (param as InParameter).Value };
                        }
                        else if (param is InOutParameter)
                        {
                            values = new object[] { param.Name, param.DataType.Name, (param as InOutParameter).Value };
                        }
                        else if (param is OutParameter)
                        {
                            values = new object[] { param.Name, param.DataType.Name, (param as OutParameter).Value };
                        }

                        this.gridViewParameter.Rows.Add(values);
                    }
                }
            }
        }

        private void CloseAllCustomScreens()
        {
            foreach (KeyValuePair<Types.ScreenHandle, CustomScreenForm> keyValuePair in this.customScreenDict)
            {
                CustomScreenForm customScreenForm = keyValuePair.Value;
                if (customScreenForm != null && !customScreenForm.IsDisposed)
                {
                    customScreenForm.Close();
                }
            }
        }

        #endregion
    }
}
