﻿//-----------------------------------------------------------------------
// <copyright file="I18NControl.cs" company="emotive GmbH & Co. KG">
//     Copyright (c) emotive GmbH & Co. KG. All rights reserved.
// </copyright>
//-----------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Windows.Forms;

namespace Emotive.Otx.Runtime.Api.Sample.Setting
{
    /// <summary>
    /// Options for the I18n
    /// </summary>
    public partial class I18NControl : ProjectOptionsPage
    {
        #region Fields
        string oldLanguageFolder;
        string oldDefaultLanguage;
        bool isChange = false;
        #endregion

        #region Ctors
        /// <summary>
        /// Initializes a new instance of the I18NOptions class.
        /// </summary>
        public I18NControl()
            : this(null)
        {
        }

        /// <summary>
        /// Initializes a new instance of the I18NOptions class.
        /// </summary>
        /// <param name="projectSetting">A ProjectConfiguration Variable</param>
        public I18NControl(ProjectSetting.ProjectSettings projectSetting)
            : base(projectSetting)
        {
            InitializeComponent();
            this.Localize();
            this.Load += new EventHandler(this.I18NOptions_Load);
        }

        #endregion

        #region Methods
        /// <summary>
        /// Overrides the SaveOptions method of the TextLibraryOptionControl class
        /// </summary>
        public override void SaveOptions()
        {
            if (!this.isChange)
            {
                return;
            }

            if (!string.IsNullOrEmpty(this.txtLanguageFolder.Text))
            {
                if (!Directory.Exists(this.txtLanguageFolder.Text))
                {
                    return;
                }
            }

            this.ProjectSetting.I18nSettings.LanguageFolder = this.txtLanguageFolder.Text;
            if (this.comboDefaultLanguage.SelectedItem != null)
            {
                this.ProjectSetting.I18nSettings.DefaultLanguage = this.comboDefaultLanguage.SelectedItem.ToString();
            }
        }

        /// <summary>
        /// Loads the data into the ComboBox when loadding the I18NOptions control
        /// </summary>
        /// <param name="sender">The I18NOptions control</param>
        /// <param name="e">The parameter contains the data event</param>
        private void I18NOptions_Load(object sender, EventArgs e)
        {
            this.oldDefaultLanguage = this.ProjectSetting.I18nSettings.DefaultLanguage;
            this.oldLanguageFolder = this.ProjectSetting.I18nSettings.LanguageFolder;
            this.txtLanguageFolder.Text = this.ProjectSetting.I18nSettings.LanguageFolder;
            FillDataToComboBox();
            this.comboDefaultLanguage.SelectedItem = this.ProjectSetting.I18nSettings.DefaultLanguage;
        }

        /// <summary>
        /// Localization text for controls
        /// </summary>
        private void Localize()
        {
            this.lblDescription.Text = "Setting of the standard text block library. The library consists of various text files within the storage folder. Each file corresponds to a language. The file format is an INI file (Section = category). An external library can be imported into the project.";
        }

        /// <summary>
        /// Shows the folderBrowserDialog and fill the data into the comboBox. 
        /// </summary>
        /// <param name="sender">The BtnBrowse button.</param>
        /// <param name="e">The parameter contains the data event.</param>
        private void BtnBrowse_Click(object sender, EventArgs e)
        {
            string folder = string.IsNullOrEmpty(this.txtLanguageFolder.Text) ? Directory.GetCurrentDirectory() : this.txtLanguageFolder.Text;

            if (!Directory.Exists(folder))
            {
                MessageBox.Show("Path does not exists.", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return;
            }

            this.folderBrowserDialog1.SelectedPath = folder;

            if (this.folderBrowserDialog1.ShowDialog() == DialogResult.OK)
            {
                string selectedPath = this.folderBrowserDialog1.SelectedPath;
                if (!this.CheckFolder(selectedPath, false))
                {
                    return;
                }

                this.txtLanguageFolder.Text = selectedPath;
            }
        }

        private void txtLanguageFolder_TextChanged(object sender, EventArgs e)
        {
            if (Directory.Exists(this.txtLanguageFolder.Text))
            {
                this.isChange = true;
                this.ProjectSetting.I18nSettings.LanguageFolder = this.txtLanguageFolder.Text;
                this.FillDataToComboBox();
            }
        }

        /// <summary>
        /// Checks the path of the folder. Checks cases as exist, empty, the valid path.
        /// </summary>
        /// <param name="path">The path of the folder</param>
        /// <param name="isCheckEmpty">If true then checks the path is empty , otherwise not checks</param>
        /// <returns>Return true if exists the folder or the valid path or the path is not empty, otherwise return false</returns>
        private bool CheckFolder(string path, bool isCheckEmpty)
        {
            string tempPath = path;
            if (isCheckEmpty)
            {
                if (string.IsNullOrEmpty(tempPath))
                {
                    MessageBox.Show("Path is null or empty.", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    return false;
                }
            }

            if (!Directory.Exists(path))
            {
                MessageBox.Show("Path does not exists.", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return false;
            }

            if (!string.IsNullOrEmpty(tempPath) && Path.GetDirectoryName(tempPath) == null)
            {
                MessageBox.Show("The path does not exist or is not a directory path. Please select a new path.", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return false;
            }
            return true;
        }

        /// <summary>
        /// Fills the list of the language into ComboBox.
        /// </summary>
        /// <param name="isNewPath">If true then reset the path of the language folder for the standardTextLibrary to gets languages</param>
        /// <param name="path">The path of the language folder</param>
        private void FillDataToComboBox()
        {
            this.comboDefaultLanguage.DataSource = null;
            this.comboDefaultLanguage.Items.Clear();
            List<string> strList = this.ProjectSetting.I18nSettings.Languages.ToList();
            strList.Sort();
            this.comboDefaultLanguage.DataSource = strList;
            if (strList.Count > 0)
            {
                this.comboDefaultLanguage.SelectedIndex = 0;
            }
        }

        public override void RollBack()
        {
            if (this.isChange)
            {
                this.ProjectSetting.I18nSettings.LanguageFolder = this.oldLanguageFolder;
                this.ProjectSetting.I18nSettings.DefaultLanguage = this.oldDefaultLanguage;
            }
        }

        private void comboDefaultLanguage_SelectedIndexChanged(object sender, EventArgs e)
        {
            this.isChange = true;
        }

        #endregion
    }
}
