﻿//-----------------------------------------------------------------------
// <copyright file="QuantitiesControl.cs" company="emotive GmbH & Co. KG">
//     Copyright (c) emotive GmbH & Co. KG. All rights reserved.
// </copyright>
//-----------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.IO;
using System.Threading;
using System.Windows.Forms;

namespace Emotive.Otx.Runtime.Api.Sample.Setting
{
    /// <summary>
    /// Displays the unit of odx files in the QuantitiesControl.
    /// </summary>
    public partial class QuantitiesControl : ProjectOptionsPage
    {
        #region Fields
        /// <summary>
        /// the list of the file should be deleted in the ODX folder. 
        /// </summary>
        private List<string> deleteFiles = new List<string>();
        private List<string> addFiles = new List<string>();

        private bool isChange = false;
        #endregion

        #region  Contructs

        /// <summary>
        /// Initializes a new instance of the QuantitiesControl class without parameters.
        /// </summary>
        public QuantitiesControl()
            : this(null)
        {
        }

        /// <summary>
        /// Initializes a new instance of the QuantitiesControl class.
        /// </summary>
        /// <param name="projectSetting">The project setting.</param>
        public QuantitiesControl(ProjectSetting.ProjectSettings projectSetting)
            : base(projectSetting)
        {
            InitializeComponent();

            this.Localize();

            Thread thread = new Thread(new ParameterizedThreadStart(delegate
                {
                    this.LoadUnitSpec();
                }));
            thread.Start();
        }
        #endregion

        #region Properties
        #endregion

        #region Methods

        /// <summary>
        /// Saves the paths of the unit file into the project config file.
        /// </summary>
        public override void SaveOptions()
        {
            if (!this.isChange)
            {
                return;
            }

            foreach (string str in this.addFiles)
            {
                this.ProjectSetting.QuantitiesSettings.AddUnitSpecFile(str);
            }

            foreach (string str in this.deleteFiles)
            {
                this.ProjectSetting.QuantitiesSettings.DeleteUnitSpecFile(str);
            }
        }

        /// <summary>
        /// Contains the displaying string of the UnitSpecControl.
        /// </summary>
        private void Localize()
        {
            this.labelDescription.Text = "The Quantities library allows you to calculate with units. The units are available in UNIT-SPEC containers from ODX. To use the units, you must refer here to the specific ODX files.";
        }

        /// <summary>
        /// Loads unitSpec list from the projectConfig file into the unitTree.
        /// </summary>
        private void LoadUnitSpec()
        {
            this.listFiles.Items.Clear();
            string[] files = this.ProjectSetting.QuantitiesSettings.UnitSpecFiles;
            this.listFiles.Items.AddRange(files);
        }

        /// <summary>
        /// Imports data into the unitTree from the odx file.
        /// </summary>
        /// <param name="sender">The object raises the event</param>
        /// <param name="e">The parameter holds the event data</param>
        private void buttonAddItem_Click(object sender, EventArgs e)
        {
            string path = this.OpenOdxFile();
            if (!string.IsNullOrEmpty(path))
            {
                this.listFiles.Items.Add(path);
                this.addFiles.Add(path);
                this.isChange = true;
            }
        }

        /// <summary>
        /// Opens a dialog and loads the file with the format *.odx.
        /// </summary>
        /// <returns>The path of the odx file</returns>
        private string OpenOdxFile()
        {
            OpenFileDialog browseFile = new OpenFileDialog();
            browseFile.Filter = "All supported formats (*.odx; *.odx-cc; *.odx-d)|*.odx;*.odx-cc;*.odx-d|ODX Files (*.odx)|*.odx|ODX-cc Files (*.odx-cc)|*.odx-cc|ODX-d Files (*.odx-d)|*.odx-d|All Files (*.*)|*.*";
            browseFile.Title = "Add ODX File";
            if (browseFile.ShowDialog() == DialogResult.OK)
            {
                string path = browseFile.FileName;
                if (!File.Exists(path))
                {
                    return string.Empty;
                }

                return path;
            }

            return string.Empty;
        }

        private void buttonDeleteItem_Click(object sender, EventArgs e)
        {
            if (this.listFiles.SelectedItem != null)
            {
                deleteFiles.Add(this.listFiles.SelectedItem.ToString());
                this.listFiles.Items.Remove(this.listFiles.SelectedItem.ToString());
                this.isChange = true;
                this.EnableButtonDelete();
            }
        }

        void EnableButtonDelete()
        {
            if (this.listFiles.SelectedIndex > -1 && this.listFiles.SelectedIndex < this.listFiles.Items.Count)
            {
                this.buttonDeleteItem.Enabled = true;
            }
            else
            {
                this.buttonDeleteItem.Enabled = false;
            }
        }

        void listFiles_SelectedIndexChanged(object sender, System.EventArgs e)
        {
            this.EnableButtonDelete();
        }

        #endregion
    }
}
